package com.sjls.nstthh.service;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import com.sjls.nstthh.entity.Grain;
import com.sjls.nstthh.entity.ReceiveData;
import com.sjls.nstthh.entity.Scantask;
import com.sjls.nstthh.entity.SendData;
import com.sjls.nstthh.repository.EquipmentRepository;
import com.sjls.nstthh.repository.GrainRepository;
import com.sjls.nstthh.repository.ReceiveDataRepository;
import com.sjls.nstthh.repository.SendDataRepository;
import com.sjls.nstthh.util.AlgAlgorithm;
import com.sjls.nstthh.util.StrSplicingAssistant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.beans.Transient;
import java.util.ArrayList;
import java.util.List;


/**
 * 通讯数据
 */

@Service
public class CommunicationService {

    private static final Logger logger = LoggerFactory.getLogger(CommunicationService.class);
    //public volatile static List receiveList = Collections.synchronizedList(new ArrayList<SendData>());
    public volatile static List receiveList = new ArrayList<SendData>();

    private SendDataRepository sendDataRepository;
    private ReceiveDataRepository receiveDataRepository;
    private GrainRepository grainRepository;
    private EquipmentRepository equipmentRepository;

    /**
     * 16进制 str
     *
     * @param z_val
     * @param y_val
     * @return
     */
    public static String send16strUseCom(long z_val, long y_val) {

        // 第一个{}为 ：z，y 角度转为步长 16进制 每隔两位字符串串加空格
        // 第二个{}为 ：sendStr_1_tmp串 crc验证后
        String sendStr_0_tmp = "681C0105000000CD96";
        String sendStr_1_tmp = "12345678031101A50102{}0012345678";
        String sendStr_2_tmp = "{}16";
//        String z = StrSplicingAssistant.hex10To16(StrSplicingAssistant.angleTransformationStep(z_val));
//        String y = StrSplicingAssistant.hex10To16(StrSplicingAssistant.angleTransformationStep(y_val));
        String z = StrSplicingAssistant.hex10To16(z_val);
        String y = StrSplicingAssistant.hex10To16(y_val);
        //16进制zy 替换 sendStr_1_tmp{}
        String sendStr_1_v = StrUtil.format(sendStr_1_tmp, z + y);
        //sendStr_1_v比特流CRC验证逆序并高低位置换后加空格格式化 替换 sendStr_2_tmp{}
        String sendStr_2_v = StrUtil.format(sendStr_2_tmp, StrSplicingAssistant.displacementHighLow(StrSplicingAssistant.CRC16Right(sendStr_1_v)));

        String sendstr_ = sendStr_0_tmp + sendStr_1_v + sendStr_2_v;
        // System.out.print(sendstr_);
        return sendstr_;
    }

    @Autowired
    public void setSendDataRepository(SendDataRepository sendDataRepository) {
        this.sendDataRepository = sendDataRepository;
    }

    @Autowired
    public void setReceiveDataRepository(ReceiveDataRepository receiveDataRepository) {
        this.receiveDataRepository = receiveDataRepository;
    }

    @Autowired
    public void setGrainRepository(GrainRepository grainRepository) {
        this.grainRepository = grainRepository;
    }

    @Autowired
    public void setEquipmentRepository(EquipmentRepository equipmentRepository) {
        this.equipmentRepository = equipmentRepository;
    }

    /**
     * 发送的
     *
     * @return
     */

    public List<SendData> getSendData(Scantask scantask) {
        List<SendData> listdb = sendDataRepository.findByCantask_id(scantask.getId());
        //logger.info("getSendData {} ",list2);
        return listdb;
    }

    /**
     * 返回的串
     *
     * @return
     */
    public List<ReceiveData> getReceiveData() {
        return receiveDataRepository.findAll();
    }

    public List<ReceiveData> getReceiveData(Scantask scantask) {
        return receiveDataRepository.findByCantask_id(scantask.getId());
    }

    /**
     * 粮食信息
     *
     * @return
     */
    public List<Grain> getGrainData(Scantask scantask) {
        return grainRepository.findByCantask_id(scantask.getId());
    }

    /**
     * 保存返回串
     *
     * @param receiveDataList
     * @return
     */
    @Transient
    public List saveReceiveData(List receiveDataList) {
        return receiveDataRepository.saveAllAndFlush(receiveDataList);
    }

    public synchronized void Pretreatment(Scantask scantask, String str_receive_head, String currentSendStr, String str_receive) {

        String currentSendStr_deviceid = currentSendStr.substring(18, 26);
        String currentSendStr_zy = currentSendStr.substring(38, 46);

        //当前接收的串，截取出角度，设备，距离。
        String deviceid = str_receive.substring(18, 26);//截取设备id
        String anglez = str_receive.substring(38, 42);//截取角度
        String angley = str_receive.substring(42, 46);//截取角度
        String anglezy = str_receive.substring(38, 46);//角度

        String distance = str_receive.substring(46, 54);//截取距离
    }
    /**
     * 验证  打点后的数据 ？是否crc 并组装到list
     *
     * @param str_receive_head
     * @param currentSendStr
     * @param str_receive
     * @return
     */
    public  boolean determineRstrPersistence(Scantask scantask, String str_receive_head, String currentSendStr, String str_receive) {
        //logger.info(" determineRstrPersistence_currentSendStr = " + currentSendStr + " str_receive= " + str_receive);
        //当前发送的串，截取出角度。
        String currentSendStr_deviceid = currentSendStr.substring(18, 26);
        String currentSendStr_zy = currentSendStr.substring(38, 46);

        //当前接收的串，截取出角度，设备，距离。
        String deviceid = str_receive.substring(18, 26);//截取设备id
        String anglez = str_receive.substring(38, 42);//截取角度
        String angley = str_receive.substring(42, 46);//截取角度
        String anglezy = str_receive.substring(38, 46);//角度

        String distance = str_receive.substring(46, 54);//截取距离

        //发送串 角度+设备 与 返回串 角度+设备 比较。
        if (currentSendStr_zy.equals(anglezy) && currentSendStr_deviceid.equals(deviceid)) {
            // 68220185000000E08812345678031101B50102029B029B0000048600FF7B123456781C7016
            String str_all = str_receive.replaceAll(str_receive_head, "");
            //返回串的待验证串
            String str_needCrc = str_all.substring(0, 50);
            //返回串的crc 4位 值
            String str_needCrc_receive = str_all.substring(50, 54);

            //crc+高低位 返回串验证
            String str_crc = StrSplicingAssistant.displacementHighLow(StrSplicingAssistant.CRC16Right(str_needCrc));

            if (str_crc.equals(str_needCrc_receive)) {
                // 根据发送与接收角度，得到返回的数据 ，并crc验证通过
                ReceiveData receiveData = new ReceiveData();
                receiveData.setReceiveStr(str_receive);
                receiveData.setReceiveKey(deviceid + anglezy);
                receiveData.setDistance(distance);
                receiveData.setCreatTime(DateTime.now());

                //16-10 - -1500 )* 0.135
                receiveData.setLevelAngle((StrSplicingAssistant.hex16To10(anglez) - 1500) * 0.135);
                receiveData.setVerticalAngle((StrSplicingAssistant.hex16To10(angley) - 1500) * 0.135);
                //16-10 -  mm-m
                double d10 = StrSplicingAssistant.hex16To10(distance);
                receiveData.setRealHypotenuseMM(d10);
                receiveData.setRealHypotenuse(d10 / 1000);
                //logger.info(receiveData + "-----------------2222222");
                receiveData.setCantask(scantask);
                receiveList.add(receiveData);
                logger.info(receiveList.size() + ": _receive_str = " + anglezy + " | " + distance);

                return true;
            } else {
                logger.info(":  _send_str_anglezy _receive_str_anglezy, crc false {} {} ", str_crc, str_needCrc_receive);
            }
        } else {
            logger.info(": _send_str_anglezy _receive_str_anglezy, val false {},{} ", currentSendStr_zy, anglezy);
        }
        return false;
    }

    /**
     * 线程 t1 打完，启动 t2
     * <p>
     * t1 完毕后，计算 体积，并最高低为基准面 再次产生待打点。
     * t2 开始。
     *
     * @return
     */
    public synchronized double againTestbyHighestPoint(Scantask scantask, List<ReceiveData> list) {

        //计算基面，创建第二次 待打点。
        double high_z = AlgAlgorithm.zHeight(list);
        //logger.info("againTestbyHighestPoint high_z  {} ", high_z);
        NRJavaSerialManager.setHigh(high_z);
        //logger.info("againTestbyHighestPoint   {} ", NRJavaSerialManager.getHigh());
        //logger.info("againTestbyHighestPoint.  AlgAlgorithm.zList {} ", AlgAlgorithm.zList);

        double v = AlgAlgorithm.calculateVolume(AlgAlgorithm.zList); //算体积 save

        String xyz = AlgAlgorithm.xyz(list, high_z);

        Grain grain = new Grain();
        grain.setGrainVolume(v);
        grain.setXyz(xyz);
        grain.setCantask(scantask);
        grainRepository.save(grain);

        return high_z;
    }

    /**
     * 创建 待打点 并 2db
     *
     * @param diameter
     * @param high
     */
    //@Transient
    public List<SendData> creatToBeTested(Scantask scantask, double diameter, double high) {

        //sendDataRepository.deleteAll();

        String str = AlgAlgorithm.getAveragePosition(diameter, high);
        String[] strArr_z_y = str.split(";");
        List<SendData> list2db = new ArrayList<SendData>();
        for (int i = 0; i < strArr_z_y.length; i++) {
            String[] strArr = strArr_z_y[i].split(",");
            //System.out.println(Long.valueOf(strArr[0]));  System.out.println(Long.valueOf(strArr[1]));
            String str10 = send16strUseCom(Long.valueOf(strArr[0]), Long.valueOf(strArr[1]));
            //System.out.println("sendStr.add(\"" + str10 + "\");");
            SendData SendData = new SendData();
            SendData.setSendStr(str10);
            String deviceid = str10.substring(18, 26);//截取设备id
            String anglezy = str10.substring(38, 46);//角度
            SendData.setSendKey(deviceid + anglezy);
            SendData.setCreatTime(DateTime.now());
            SendData.setCantask(scantask);
            list2db.add(SendData);
        }

        List listdb = sendDataRepository.saveAll(list2db);

        return listdb;
    }

}
