package com.sjls.nstthh.service;

import com.sjls.nstthh.entity.*;
import com.sjls.nstthh.repository.ScantaskRepository;
import com.sjls.nstthh.util.AlgAlgorithm;
import com.sjls.nstthh.util.StrSplicingAssistant;
import gnu.io.NRSerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.beans.Transient;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;

@Component
public class NRJavaSerialManager {

    private static final Logger logger = LoggerFactory.getLogger(NRJavaSerialManager.class);
    private static final String str_init = "681C0105000000CD9612345678031101A5010205DC05DC00123456785FA816";
    private static final String str_heartbeat = "68140186000000E55A1234567803510112345678B1E216";
    private static final String str_receive_head = "68220185000000E088";
    volatile static double high = 31;
    volatile static double diameter = 12;
    volatile static double h1 = 6;
    volatile static double h2 = 3;
    static String cloud_url = "http://192.168.1.216:9025/api/n-basic-edge/getEdgeData";
    private volatile static String currentSendStr = str_init;
    //private volatile static List<SendData> sendData = Collections.synchronizedList(new ArrayList<SendData>());
    private volatile static List<SendData> sendData = new ArrayList<SendData>();

    private volatile static boolean isHeartbeat = false;
    private volatile static boolean isReceive = false;
    private volatile static boolean isSerialPortDisconnect = false;
    private static Scantask scantask;
    private static CommunicationService communicationService;
    private static ScantaskRepository scantaskRepository;

    @Autowired
    public void setScantaskRepository(ScantaskRepository scantaskRepository) {
        this.scantaskRepository = scantaskRepository;
    }

    public static double getHigh() {
        return high;
    }

    public static void setHigh(double high) {
        NRJavaSerialManager.high = high;
    }

    public static double getDiameter() {
        return diameter;
    }

    public static void setDiameter(double diameter) {
        NRJavaSerialManager.diameter = diameter;
    }

    public static double getH1() {
        return h1;
    }

    public static void setH1(double h1) {
        NRJavaSerialManager.h1 = h1;
    }

    public static double getH2() {
        return h2;
    }

    public static void setH2(double h2) {
        NRJavaSerialManager.h2 = h2;
    }

    public final static boolean wait10Seconds(long m) {
        try {
            Thread.sleep(m);
        } catch (InterruptedException e) {
        }
        return true;
    }

    @Transient
    public static void synctoCloudData() throws UnsupportedEncodingException {

        List<Grain> grainList = communicationService.getGrainData(scantask);
        double realHypotenuse_avg = getVolumeAvg();

        double v = 0;
        String str = "";
        for (Grain grain : grainList) {
            v += grain.getGrainVolume();
            str += grain.getXyz();
        }

        SyncV2xyz syncV2xyz = new SyncV2xyz();
        syncV2xyz.setXyz(URLEncoder.encode(str, "UTF-8"));
        syncV2xyz.setVolume(v / 2);
        syncV2xyz.setVolume_2all(realHypotenuse_avg);
        syncV2xyz.setStartTime(scantask.getCreatTime());

        //任务完成
        scantask.setOver(true);
        Scantask scantaskdb = scantaskRepository.saveAndFlush(scantask);
        logger.info("Grain -  {},{}", scantaskdb, syncV2xyz);

        RestTemplate restTemplate = new RestTemplate();
        try {
            restTemplate.postForObject(cloud_url, syncV2xyz, String.class);
        } catch (Exception e) {
        }
    }

    /**
     * 两次打点数据 平均
     *
     * @throws
     */
    public static double getVolumeAvg() {
        //查询所有打点记录。
        List<ReceiveData> receiveDataList = communicationService.getReceiveData(scantask);
        // 组装 zlist
        List<Double> zDataList =  AlgAlgorithm.getZList(receiveDataList);
        //算体积
        double realHypotenuse_avg = AlgAlgorithm.calculateVolume(zDataList);

        return realHypotenuse_avg;
    }

    public void setScantask(Scantask scantask) {
        NRJavaSerialManager.scantask = scantask;
    }

    @Autowired
    public void setCommunicationService(CommunicationService communicationService) {
        NRJavaSerialManager.communicationService = communicationService;
    }

    /**
     * 启动时，打两次。
     *
     * @param serialPort
     * @throws IOException
     * @throws TooManyListenersException
     * @throws InterruptedException
     */
    public void operation(NRSerialPort serialPort) throws IOException, InterruptedException {

        DataOutputStream outStream = null;
        if (serialPort != null && serialPort.isConnected()) {
            DataInputStream inStream = new DataInputStream(serialPort.getInputStream());
            try {
                serialPort.addEventListener(new SerialReader(inStream));
            } catch (TooManyListenersException e) {
            }
            serialPort.notifyOnDataAvailable(true);
            outStream = new DataOutputStream(serialPort.getOutputStream());

        }
        Thread t1 = new Thread(new SerialWriter(outStream), "t1_thread");
        t1.start();
        t1.join();
        //wait10Seconds(5000);
        Thread t2 = new Thread(new SerialWriter(outStream), "t2_thread");
        t2.start();
        t2.join();
        //wait10Seconds(5000);
        Thread t3 = new Thread(() -> {
            try {
                synctoCloudData();
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        });
        t3.start();
    }

    public static class SerialReader implements SerialPortEventListener {
        private final DataInputStream inStream;
        private String str_receive_at = "";

        public SerialReader(DataInputStream inStream) {
            this.inStream = inStream;
        }

        public void serialEvent(SerialPortEvent arg0) {
            try {
                byte[] buffer = new byte[inStream.available()];
                while (inStream.read(buffer) != -1) {
                    break;
                }
                String str_receive = StrSplicingAssistant.bytesToHexString(buffer);

                if (str_receive.equals(str_heartbeat)) {
                    isHeartbeat = true;
                } else {
                    if (str_receive.length() == 64) {
                        str_receive_at = str_receive;
                    }
                    if (str_receive.length() == 10) {
                        str_receive_at += str_receive;
                    }

                    if (str_receive_at.length() == 74 && !isReceive) {
                        //logger.info(" 74 isReceive str_receive_at {} {} {} ", isHeartbeat, isReceive, str_receive_at);

                        String currentSendStr_deviceid = currentSendStr.substring(18, 26);
                        String currentSendStr_zy = currentSendStr.substring(38, 46);
                        //当前接收的串，截取出角度，设备，距离。
                        String deviceid = str_receive_at.substring(18, 26);//截取设备id
                        String anglezy = str_receive_at.substring(38, 46);//角度
                        //logger.info(" 74 isReceive currentSendStr_zy,anglezy {} {} {} ",currentSendStr_zy ,anglezy ,str_receive_at);

                        //发送串 角度+设备 与 返回串 角度+设备 比较。
                        if (currentSendStr_zy.equals(anglezy) && currentSendStr_deviceid.equals(deviceid)) {
                            //synSet.add(str_receive_at);
                            if (!str_receive_at.contains("EEEEEEEE")) {
                                // 判断返回串 正确与否，并组装 list for add。
                                isReceive = communicationService.determineRstrPersistence(scantask, str_receive_head, currentSendStr, str_receive_at);
                                // logger.info(" while - str_receive_at = {}, {} ", str_receive_at, isReceive);
                            }
                            //isReceive = true;
                            // break;
                        }
                    }
                }

            } catch (IOException e) {
                e.printStackTrace();
                return;
            }
        }
    }

    public static class SerialWriter implements Runnable {
        private final DataOutputStream outStream;
        //初始化设备位置，只一次。
        boolean onlyOnce = true;

        public SerialWriter(DataOutputStream outStream) {
            this.outStream = outStream;
        }

        public void run() {
            sendData = communicationService.creatToBeTested(scantask, diameter, high);
            while (sendData.size() > 0) {
                break;
            }
            logger.info("1.15 -15 Beta version wait for send point number testing ......" + sendData.size());

            //没心跳的话，等待10次。
            int count = 0;
            while (true) {
                if (isHeartbeat && onlyOnce || count == 10) {
                    try {
                        outStream.write(StrSplicingAssistant.hexStrToByteArrs(str_init));
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    onlyOnce = false;
                    isReceive = false;
                    logger.info("init_send_str =" + str_init.substring(38, 46));
                    break;
                }
                count++;
                wait10Seconds(2000);
            }

            while (!Thread.interrupted()) {
                try {
//                        if (!isReceive && onlyOnce) {
//                            if (sendData != null && sendData.size() > 0)
//                                outStream.write(StrSplicingAssistant.hexStrToByteArrs(String.valueOf(sendData.get(0))));
//                            wait10Seconds(8000);
//                            isReceive = true;
//                        }
                    if (isHeartbeat && isReceive || wait10Seconds(10000)) {
                        if (sendData != null && sendData.size() > 0) {
                            Iterator<SendData> iterator = sendData.iterator();
                            if (iterator.hasNext()) {
                                //synchronized (this) {
                                currentSendStr = iterator.next().getSendStr();
                                outStream.write(StrSplicingAssistant.hexStrToByteArrs(currentSendStr));
                                logger.info("_send_str = {}", currentSendStr.substring(38, 46));
                                iterator.remove();
                                isReceive = false;
                            }
                        } else {
                            List receiveList_ = CommunicationService.receiveList;
                            // 组装的list 一次性save。
                            List receiveListDB_ = communicationService.saveReceiveData(receiveList_);
                            wait10Seconds(2000);
                            if (receiveListDB_ != null && receiveListDB_.size() > 0) {
                                isSerialPortDisconnect = true;
                                logger.info(isSerialPortDisconnect + "_success _ :" + Thread.currentThread().getName());
                                high = communicationService.againTestbyHighestPoint(scantask, receiveListDB_);
                                isReceive = false;
                                isHeartbeat = false;
                                Thread.currentThread().interrupt();
                                return;
                            }
                        }
                    }
                    wait10Seconds(6000);
                } catch (IOException e) {
                    e.printStackTrace();
                    return;
                }
            }
        }

    }
}