package com.sjls.nstthh.util;


import com.sjls.nstthh.entity.ReceiveData;
import com.sjls.nstthh.service.NRJavaSerialManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class AlgAlgorithm {
    private static final Logger logger = LoggerFactory.getLogger(AlgAlgorithm.class);

    //public volatile static List<Double> zList = Collections.synchronizedList(new ArrayList<Double>()); //粮面高的集合
    public volatile static List<Double> zList = new ArrayList<Double>(); //粮面高的集合

    //圆筒仓的直径
    static double diameter = NRJavaSerialManager.getDiameter();
    //high 是设备到地面的高即AC
    static double high = NRJavaSerialManager.getHigh();

    static double h1 = NRJavaSerialManager.getH1(); //h1 圆锥的高
    static double h2 = NRJavaSerialManager.getH2(); // h2 水平地面到圆锥尖的高

    /**
     * 算法 产生 待检测点。
     *
     * @param high 高
     * @return
     */
    public static String getAveragePosition(double diameter, double high) {

        if (diameter == 0) {
            diameter = diameter;
        }
        if (high == 0) {
            high = high;
        }
        StringBuffer sb = new StringBuffer();

        double x1 = 0;
        double y1 = 0;
        //打出来三角形的底边AB 实际是平面上点的斜边
        double AB = 0; //理想状态
        double x = 0;
        double y = 0;
        double BC = 0;
        double vertical_angle = 0;//垂直的角度
        double level_angle = 0;//水平的角度
        double vertical_step = 0;//垂直的步长
        double level_step = 0;//水平的步长

        //根据直径判断半径是整数还是非整数 如果是非整数 则向下取整
        double r = Math.floor(diameter / 2);  //最大的半径 即一共有多少个圆圈

        //一共有多少个圆圈
        int circleNumber = (int) r;

        for (int i = 1; i <= circleNumber; i++) {
            //每一个圆一共有多少个的点   此时的半径长就是i
            int pointCount = (int) ((2 * Math.PI * i) / 1.57);

            for (int j = 0; j < pointCount; j++) {

                /**
                 * 以其他点为圆心的圆  r是i degrees是以圆心为中心转动的角度 其坐标x1、y1则是
                 */
                double degrees = (360 / pointCount) * j;
                //System.out.println(degrees+"度数");
                double radians = Math.toRadians(degrees);//转化为弧度
                String strx = String.format("%.2f%n", Math.cos(radians) * i);//计算横坐标的值
                x1 = new Double(strx);
                String stry = String.format("%.2f%n", Math.sin(radians) * i);//计算纵坐标的值
                y1 = new Double(stry);

                /**
                 * 以设备为中心，把圆上的点分为上下两半，
                 * 新的圆 是旧的圆向左平移了(diameter/2-i)+i 个长度 得到的
                 * (设备在新的圆的最左边上，设备是坐标系的原点)
                 * 所以 横坐标加上这个长度  纵坐标不变
                 */

                //纵坐标 无论是处于上半部分还是下半部分 其纵坐标是不变的
                x = x1 + (diameter / 2 - i) + i;
                y = y1;
                //System.out.print("["+x+","+y+"]"+",");
                //底面三角形的斜边
                AB = Math.sqrt(x * x + y * y);
                //垂直三角形的斜边
                BC = Math.sqrt(AB * AB + high * high);
                if (x == 0 && y == 0) {//圆 点
                    level_angle = 0;
                    vertical_angle = 0;

                } else if (y < 0) {//反向转
                    level_angle = -Math.toDegrees(Math.acos(Math.abs(y) / AB));
                    vertical_angle = -(Math.toDegrees(Math.acos(high / BC)));

                } else {//正向转
                    level_angle = Math.toDegrees(Math.acos(y / AB));
                    vertical_angle = Math.toDegrees(Math.acos(high / BC));

                }
                //转为步长
                level_step = 1500 + level_angle / 0.135;
                vertical_step = 1500 + vertical_angle / 0.135;

                sb.append(Math.round(level_step));
                sb.append(",");
                sb.append(Math.round(vertical_step));
                sb.append(";");
                // sb.append("\n");
            }
        }

        /**
         * 追加一个圆筒仓中心的点，其坐标是(diameter/2,0) 水平转动90度 步长为2167
         */
        AB = diameter / 2;//垂直三角形的底边
        BC = Math.sqrt(AB * AB + high * high);//垂直三角形的斜边
        vertical_angle = 90 - Math.toDegrees(Math.acos(high / BC));
        vertical_step = 1500 + vertical_angle / 0.135;
        sb.append(2167);
        sb.append(",");
        sb.append(Math.round(vertical_step));
        // sb.append("\n");
        //System.out.println(Math.round(level_step)+","+Math.round(vertical_step)+"");
        //System.out.println(sb);
        return sb.toString();
    }


    /**
     * 激光测距 给三维的坐标
     *
     * @param list(水平角度、垂直角度、返回的距离)
     * @param high(设备到基准面的高)
     * @return
     */
    public static String xyz(List<ReceiveData> list, double high) {

        //double diameter = 30;//圆筒仓的直径

        double AC = high;
        double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面到设备的水平距离
        double x = 0; //实际点的坐标的x轴
        double y = 0; //实际点的坐标的y轴
        double z = 0; //实际点的坐标的z轴
        int yy = 0;
        int xx = 0;
        StringBuffer sb = new StringBuffer();
        sb.append("[");

        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离
        for (ReceiveData angleObject : list) {
            verticalAngle = angleObject.getVerticalAngle();
            levelAngle = angleObject.getLevelAngle();
            realHypotenuse = angleObject.getRealHypotenuse();

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinc = new Double(verticalAngleS);
            //B1C = realHypotenuse;
            A1B1 = realHypotenuse * sinc; //即 平面三角形的斜边
            String sinLevel = String.format("%.2f%n", Math.sin(Math.abs(levelAngle)));
            double sinLevelc = new Double(sinLevel);


            /**
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             */
            //打到墙上 最大的边
            double maxLength = sinLevelc * diameter;

            if (A1B1 < maxLength) {
                //实际打在粮面上的点的坐标
                String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
                double cosc = new Double(verticalAngleC);
                double A1C = B1C * cosc;
                z = AC - A1C;

                //平面三角形中 已知斜边A1B1和一个角(水平转动的角 angleObject1.getLevelAngle()) 求两直角边
                double LevelAngleRadians = Math.toRadians(Math.abs(levelAngle));
                String LevelAngleS = String.format("%.2f%n", Math.sin(LevelAngleRadians));
                double sinc1 = new Double(LevelAngleS);
                x = sinc1 * A1B1;

                String LevelAngleC = String.format("%.2f%n", Math.cos(LevelAngleRadians));
                double cosc1 = new Double(LevelAngleC);
                y = cosc1 * A1B1;

                //发送给三维的坐标 需要向上平移d/2个单位
                if (levelAngle > 0) {
                    //点的坐标就是 (x,y,z)

                } else if (levelAngle < 0) {
                    //点的坐标就是 (x,-y,z)
                    y = -y;
                }
                y = y + diameter / 2;


                BigDecimal big_z = new BigDecimal(z);
                z = big_z.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();

                yy = (int) Math.round(y);
                xx = (int) Math.round(x);

                sb.append("[" + xx + "," + yy + "," + z + "],");
                //sb.append("[" + x + "," + y + "," + z + "],");
            }

        }
        sb = sb.deleteCharAt(sb.length() - 1);
        sb.append("]");
        return sb.toString();

    }

    /**
     * 以粮面为基准面的高
     *
     * @param list(水平角度、垂直角度、返回的距离)
     * @param high(设备到基准面的高)
     * @return
     */
    public static double zHeight(List<ReceiveData> list) {
        if (list == null && list.size() == 0) {
            return high;
        }
        // logger.info("zHeight - list",list);

        double AC = high;
        double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面到设备的水平距离
        double z = 0; //实际点的坐标的z轴
        double zHigh = 0;//以粮面为基准面的高
        double zMax = 0;//最高的点
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离
        for (ReceiveData angleObject : list) {
            verticalAngle = angleObject.getVerticalAngle();
            levelAngle = angleObject.getLevelAngle();
            realHypotenuse = angleObject.getRealHypotenuse();
            //logger.info("zHeight - realHypotenuse {} ",realHypotenuse);

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinc = new Double(verticalAngleS);
            //B1C = realHypotenuse;
            A1B1 = realHypotenuse * sinc; //即 平面三角形的斜边

            //logger.info("zHeight - A1B1 {} ",A1B1);

            String sinLevel = String.format("%.2f%n", Math.sin(Math.abs(levelAngle)));
            double sinLevelc = new Double(sinLevel);

            /**
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             */
            //打到墙上 最大的边
            double maxLength = sinLevelc * diameter;

            if (A1B1 < maxLength) {
                //实际打在粮面上的点的坐标
                String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
                double cosc = new Double(verticalAngleC);
                double A1C = realHypotenuse * cosc;
                z = AC - A1C;


                // logger.info("zHeight - z√√√√√√z z z z z {} ",z);

                zList.add(z);
            }
        }
        zMax = Collections.max(zList);
        zHigh = AC - zMax;
        return zHigh;

    }


    public static List<Double> getZList(List<ReceiveData> list) {
        if (list == null && list.size() == 0) {
            return zList;
        }
        // logger.info("zHeight - list",list);

        double AC = high;
        double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面到设备的水平距离
        double z = 0; //实际点的坐标的z轴
        double zHigh = 0;//以粮面为基准面的高
        double zMax = 0;//最高的点
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离
        for (ReceiveData angleObject : list) {
            verticalAngle = angleObject.getVerticalAngle();
            levelAngle = angleObject.getLevelAngle();
            realHypotenuse = angleObject.getRealHypotenuse();
            //logger.info("zHeight - realHypotenuse {} ",realHypotenuse);

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinc = new Double(verticalAngleS);
            //B1C = realHypotenuse;
            A1B1 = realHypotenuse * sinc; //即 平面三角形的斜边

            //logger.info("zHeight - A1B1 {} ",A1B1);

            String sinLevel = String.format("%.2f%n", Math.sin(Math.abs(levelAngle)));
            double sinLevelc = new Double(sinLevel);

            /**
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             */
            //打到墙上 最大的边
            double maxLength = sinLevelc * diameter;

            if (A1B1 < maxLength) {
                //实际打在粮面上的点的坐标
                String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
                double cosc = new Double(verticalAngleC);
                double A1C = realHypotenuse * cosc;
                z = AC - A1C;


                // logger.info("zHeight - z√√√√√√z z z z z {} ",z);

                zList.add(z);
            }
        }

        return zList;

    }

    /**
     * 计算体积
     *
     * @param zList 粮面高集合(z值的集合)
     */
    public static double calculateVolume(List<Double> zList) {

        //17.1  18.3  19.2  18.3    20.1  16.4  19.8  16.7  19.0  18.9
        Double zMin = Collections.min(zList); //z的最小值

        double sum = 0;
        double zHeigh = 0;//每一个z与最低点的z差值
        for (int i = 0; i < zList.size(); i++) {
            //sum = zList.get(i)+sum;
            zHeigh = zList.get(i) - zMin;
            sum += zHeigh;

        }
        /** v 是总体积
         * v1 最上面的那一部分
         * v2 是中间的圆柱
         * v3 是 圆锥
         *
         * h1 圆锥的高
         * h2 水平地面到圆锥尖的高
         */

        double v1 = 3.14 * (diameter / 2) * (diameter / 2) * (sum / zList.size());
        double v2 = 3.14 * (diameter / 2) * (diameter / 2) * (zMin - h2 - h1);
        double v3 = (3.14 * (diameter / 2) * (diameter / 2) * h1) / 3;
        double vv = v1 + v2 + v3;
        BigDecimal bdv = new BigDecimal(vv);
        double v = bdv.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        return v;
    }

}
