package com.sjls.nstthh.util;

import java.math.BigInteger;

public class StrSplicingAssistant {

    /**
     * 10进制 转 16进制
     *
     * @param i
     * @return 16_i
     */

    private static final String hexString = "0123456789ABCDEF";

    /**
     * 逆序校验
     *
     * @param dataHexStr
     * @return
     */
    public static String CRC16Right(String dataHexStr) {
        //字符串类型的16进制字符串转换为字节数组
        byte[] data = hexStrToByteArrs(dataHexStr);
        //初始化寄存器 我们用数据的低八位进行处理,已经可以不用限制寄存器的个数了
        //我测试了int long 都能达到预期的效果 但是short 反而数据错误 没想明白为什么。
        int crci = 0xffff;
        int i, j;
        for (j = 0; j < data.length; j++) {
            //此处&0x00ff的目的是为了将数据放入寄存器的低8位并与寄存器异或
            crci ^= data[j] & 0x00ff;
            for (i = 0; i < 8; i++) {
                //判断即将右移的是不是1 如果是1则与多项式进行异或
                if ((crci & 0X0001) == 0X0001) {
                    crci >>= 1;
                    //将数据与约定的多项式做（模二除/异或）  数据流向变了此处的异或值实际是8005的逆序反转a001
                    crci ^= 0xa001;
                } else {
                    crci >>= 1;
                }
            }
        }
        //结果值与0x0000异或 其实值不会改变，不异或对结果没有影响
        String res = Long.toHexString(crci ^ 0x0000);

        while (res.length() < 4) {
            res = "0" + res;
        }
        return res;
    }

    /**
     * 将十六进制的字符串转换成字节数组
     *
     * @param hexString
     * @return byte[]
     */
    public static byte[] hexStrToByteArrs(String hexString) {
        hexString = hexString.replaceAll(" ", "");
        int len = hexString.length();
        int index = 0;

        byte[] bytes = new byte[len / 2];

        while (index < len) {
            String sub = hexString.substring(index, index + 2);
            bytes[index / 2] = (byte) Integer.parseInt(sub, 16);
            index += 2;
        }

        return bytes;
    }

    /**
     * 数组转换成十六进制字符串
     *
     * @param bArray
     * @return HexString
     */
    public static final String bytesToHexString(byte[] bArray) {

        StringBuffer sb = new StringBuffer(bArray.length);
        String sTemp;
        for (int i = 0; i < bArray.length; i++) {
            sTemp = Integer.toHexString(0xFF & bArray[i]);
            if (sTemp.length() < 2)
                sb.append(0);
            sb.append(sTemp.toUpperCase());
        }
        return sb.toString().trim().toUpperCase();
    }

    /*显示完美*/
    public static final String toHex(byte[] data, int off, int length) {
        // double size, two bytes (hex range) for one byte
        StringBuffer buf = new StringBuffer(data.length * 2);
        for (int i = off; i < length; i++) {
            // don't forget the second hex digit
            if (((int) data[i] & 0xff) < 0x10) {
                buf.append("0");
            }
            buf.append(Long.toString((int) data[i] & 0xff, 16));
            if (i < data.length - 1) {
                buf.append(" ");
            }
        }
        return buf.toString().trim().toUpperCase();
    }

    /**
     * 10进制 转 16进制
     *
     * @param i
     * @return 16_i
     */
    public static String hex10To16(long i) {
        return String.format("%04X", i);
    }

    /**
     * @param strHex
     * @return
     */
    public static int hex16To10(String strHex) {
        BigInteger bigInteger = new BigInteger(strHex, 16);
        return bigInteger.intValue();
    }


    /**
     * 高低位转换 大写
     * 如“1A2B”转“2B1A”。
     *
     * @param hex
     * @return String
     */
    public static String displacementHighLow(String hex) {
        char[] charArray = hex.toCharArray();
        int length = charArray.length;
        int times = length / 2;
        for (int c1i = 0; c1i < times; c1i += 2) {
            int c2i = c1i + 1;
            char c1 = charArray[c1i];
            char c2 = charArray[c2i];
            int c3i = length - c1i - 2;
            int c4i = length - c1i - 1;
            charArray[c1i] = charArray[c3i];
            charArray[c2i] = charArray[c4i];
            charArray[c3i] = c1;
            charArray[c4i] = c2;
        }
        return new String(charArray).toUpperCase();

    }
}
